package com.lovecoding.redis.redisdemo;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;
import org.springframework.web.filter.OncePerRequestFilter;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.Objects;

@Component
public class SpringSecurityFilter  extends OncePerRequestFilter {

    @Autowired
    RedisService redisService;

    /**
     * 自己建过滤器 是为了 整合 JWT
     * Security 就是一个基于 过滤器 实现的 权限控制框架
     * 第一点 我们要自己鉴权控制权限
     * 我们还要使用 Security 的权限管理
     * 我们必须要保证 我们的过滤器 优先执行
     * @param request
     * @param response
     * @param filterChain
     * @throws ServletException
     * @throws IOException
     */
    @Override
    protected void doFilterInternal(HttpServletRequest request, HttpServletResponse response, FilterChain filterChain) throws ServletException, IOException {

        //从 Header取 token
        String token = request.getHeader("token");

        //如果header没有token则从URL取
        if ( !StringUtils.hasText( token ) ) {
            token = request.getParameter("token");
        }

        //如果没有token 则放行不处理
        if ( !StringUtils.hasText( token )) {
            filterChain.doFilter( request, response );
            return;
        }

        //从 redis里 查询用户信息
        Object o = redisService.get("token:" + token);

        //如果能查到用户信息 则认为用户已登录 直接给 Security 赋值权限
        if (Objects.nonNull( o )) {
            //用户存在
            UsernamePasswordAuthenticationToken userToken =
                    new UsernamePasswordAuthenticationToken( "username", null, null );
            SecurityContextHolder.getContext().setAuthentication( userToken );
        } else {
            //如果 redis 查不到 用户信息 则认为 登录过期 清空Security上下文
            SecurityContextHolder.clearContext();
        }

        //过滤器放心
        filterChain.doFilter( request, response );

    }
}
